package co.yixiang.yshop.module.distributor.service.distributor;

import java.math.BigDecimal;
import java.util.*;
import javax.validation.*;

import co.yixiang.yshop.module.distributor.controller.admin.distributor.vo.*;
import co.yixiang.yshop.module.distributor.dal.dataobject.distributor.DistributorDO;
import co.yixiang.yshop.framework.common.pojo.PageResult;

/**
 * 分销商 Service 接口
 *
 * @author admin
 */
public interface DistributorService {

    /**
     * 创建分销商
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long create(@Valid DistributorCreateReqVO createReqVO);

    Long save(DistributorDO distributorDO);

    /**
     * 更新分销商
     *
     * @param updateReqVO 更新信息
     */
    void update(@Valid DistributorUpdateReqVO updateReqVO);

    void update(DistributorDO distributorDO);

    /**
     * 删除分销商
     *
     * @param id 编号
     */
    void delete(Long id);

    /**
     * 获得分销商
     *
     * @param id 编号
     * @return 分销商
     */
    DistributorDO get(Long id);

    /**
     * 获得分销商列表
     *
     * @param ids 编号
     * @return 分销商列表
     */
    List<DistributorDO> getList(Collection<Long> ids);

    /**
     * 获得分销商分页
     *
     * @param pageReqVO 分页查询
     * @return 分销商分页
     */
    PageResult<DistributorDO> getPage(DistributorPageReqVO pageReqVO);

    /**
     * 获得分销商列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 分销商列表
     */
    List<DistributorDO> getList(DistributorExportReqVO exportReqVO);

    DistributorDO getDistributorByUserId(Long userId);

    void clearance(Long id);

    void addUpDistributor(Long id);

    void addAmount(Long id, BigDecimal productPrice, BigDecimal addUpWages);
}
