package co.yixiang.yshop.module.distributor.service.distributororder;

import java.time.LocalDateTime;
import java.util.*;
import javax.validation.*;

import co.yixiang.yshop.module.distributor.controller.admin.distributororder.vo.*;
import co.yixiang.yshop.module.distributor.dal.dataobject.distributororder.DistributorOrderDO;
import co.yixiang.yshop.framework.common.pojo.PageResult;

/**
 * 分销商订单 Service 接口
 *
 * @author admin
 */
public interface DistributorOrderService {

    /**
     * 创建分销商订单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOrder(@Valid DistributorOrderCreateReqVO createReqVO);

    /**
     * 更新分销商订单
     *
     * @param updateReqVO 更新信息
     */
    void updateOrder(@Valid DistributorOrderUpdateReqVO updateReqVO);

    /**
     * 删除分销商订单
     *
     * @param id 编号
     */
    void deleteOrder(Long id);

    /**
     * 获得分销商订单
     *
     * @param id 编号
     * @return 分销商订单
     */
    DistributorOrderDO getOrder(Long id);

    DistributorOrderDO getOrderByDetailId(Long detailId);

    /**
     * 获得分销商订单列表
     *
     * @param ids 编号
     * @return 分销商订单列表
     */
    List<DistributorOrderDO> getOrderList(Collection<Long> ids);

    /**
     * 获得分销商订单分页
     *
     * @param pageReqVO 分页查询
     * @return 分销商订单分页
     */
    PageResult<DistributorOrderDO> getOrderPage(DistributorOrderPageReqVO pageReqVO);

    /**
     * 获得分销商订单列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 分销商订单列表
     */
    List<DistributorOrderDO> getOrderList(DistributorOrderExportReqVO exportReqVO);

    /**
     * 获取佣金金额
     *
     * @param id
     * @return
     */
    DistributorOrderDO sumFirstAmount(Long id, LocalDateTime[] createTime, Integer status);

    DistributorOrderDO sumSecondAmount(Long id, LocalDateTime[] createTime, Integer status);

    void insertBatch(List<DistributorOrderDO> list);
}
