package co.yixiang.yshop.module.order.service.storeorderdetail;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.order.controller.admin.storeorderdetail.vo.StoreOrderDetailCreateReqVO;
import co.yixiang.yshop.module.order.controller.admin.storeorderdetail.vo.StoreOrderDetailExportReqVO;
import co.yixiang.yshop.module.order.controller.admin.storeorderdetail.vo.StoreOrderDetailPageReqVO;
import co.yixiang.yshop.module.order.controller.admin.storeorderdetail.vo.StoreOrderDetailUpdateReqVO;
import co.yixiang.yshop.module.order.dal.dataobject.storeorder.StoreOrderDO;
import co.yixiang.yshop.module.order.dal.dataobject.storeorderdetail.StoreOrderDetailDO;
import co.yixiang.yshop.module.product.controller.app.cart.vo.AppStoreCartQueryVo;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 订单明细 Service 接口
 *
 * @author moxiangrong
 */
public interface StoreOrderDetailService {

    /**
     * 创建订单明细
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createStoreOrderDetail(@Valid StoreOrderDetailCreateReqVO createReqVO);

    /**
     * 更新订单明细
     *
     * @param updateReqVO 更新信息
     */
    void updateStoreOrderDetail(@Valid StoreOrderDetailUpdateReqVO updateReqVO);

    /**
     * 删除订单明细
     *
     * @param id 编号
     */
    void deleteStoreOrderDetail(Long id);

    /**
     * 获得订单明细
     *
     * @param id 编号
     * @return 订单明细
     */
    StoreOrderDetailDO getStoreOrderDetail(Long id);

    /**
     * 获得订单明细列表
     *
     * @param ids 编号
     * @return 订单明细列表
     */
    List<StoreOrderDetailDO> getStoreOrderDetailList(Collection<Long> ids);

    /**
     * 获得订单明细分页
     *
     * @param pageReqVO 分页查询
     * @return 订单明细分页
     */
    PageResult<StoreOrderDetailDO> getStoreOrderDetailPage(StoreOrderDetailPageReqVO pageReqVO);

    /**
     * 获得订单明细列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 订单明细列表
     */
    List<StoreOrderDetailDO> getStoreOrderDetailList(StoreOrderDetailExportReqVO exportReqVO);

    /**
     * 批量保存
     *
     * @param oid        订单id
     * @param orderId    订单编号
     * @param cartInfo   购物车信息
     * @param storeOrder 订单信息
     */
    void batchInsert(Long oid, String orderId, List<AppStoreCartQueryVo> cartInfo, StoreOrderDO storeOrder);


    /**
     * 修改明细状态，并获取订单下一步状态
     *
     * @param afterSalesId
     * @param oid
     * @return
     */
    Integer updateState(Long afterSalesId, Long oid);

}
