package co.yixiang.yshop.module.product.service.shippingtemplates;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.product.controller.admin.shippingtemplates.vo.ShippingTemplatesCreateReqVO;
import co.yixiang.yshop.module.product.controller.admin.shippingtemplates.vo.ShippingTemplatesExportReqVO;
import co.yixiang.yshop.module.product.controller.admin.shippingtemplates.vo.ShippingTemplatesPageReqVO;
import co.yixiang.yshop.module.product.controller.admin.shippingtemplates.vo.ShippingTemplatesUpdateReqVO;
import co.yixiang.yshop.module.product.dal.dataobject.shippingtemplates.ShippingTemplatesDO;
import co.yixiang.yshop.module.product.service.shippingtemplates.dto.ShippingTemplatesDTO;
import com.baomidou.mybatisplus.extension.service.IService;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 运费模板 Service 接口
 *
 * @author yshop
 */
public interface ShippingTemplatesService extends IService<ShippingTemplatesDO> {

    /**
     * 创建运费模板
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Integer createShippingTemplates(@Valid ShippingTemplatesCreateReqVO createReqVO);

    /**
     * 更新运费模板
     *
     * @param updateReqVO 更新信息
     */
    void updateShippingTemplates(@Valid ShippingTemplatesUpdateReqVO updateReqVO);

    /**
     * 删除运费模板
     *
     * @param id 编号
     */
    void deleteShippingTemplates(Integer id);

    /**
     * 获得运费模板
     *
     * @param id 编号
     * @return 运费模板
     */
    ShippingTemplatesDO getShippingTemplates(Integer id);

    /**
     * 获得运费模板列表
     *
     * @param ids 编号
     * @return 运费模板列表
     */
    List<ShippingTemplatesDO> getShippingTemplatesList(Collection<Integer> ids);

    /**
     * 获得运费模板分页
     *
     * @param pageReqVO 分页查询
     * @return 运费模板分页
     */
    PageResult<ShippingTemplatesDO> getShippingTemplatesPage(ShippingTemplatesPageReqVO pageReqVO);

    /**
     * 获得运费模板列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 运费模板列表
     */
    List<ShippingTemplatesDO> getShippingTemplatesList(ShippingTemplatesExportReqVO exportReqVO);

    /**
     * 新增与更新模板
     *
     * @param id                   模板id
     * @param shippingTemplatesDto ShippingTemplatesDto
     */
    void addAndUpdate(Integer id, ShippingTemplatesDTO shippingTemplatesDto);

}
