package co.yixiang.yshop.module.shop.service.canvas;

import java.util.*;
import javax.validation.*;

import co.yixiang.yshop.module.shop.controller.admin.canvas.vo.*;
import co.yixiang.yshop.module.shop.controller.admin.canvasdetail.vo.CanvasDetailRespVO;
import co.yixiang.yshop.module.shop.dal.dataobject.canvas.CanvasDO;
import co.yixiang.yshop.framework.common.pojo.PageResult;

/**
 * 画布信息 Service 接口
 *
 * @author moxiangrong
 */
public interface CanvasService {

    /**
     * 创建画布信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCanvas(@Valid CanvasCreateReqVO createReqVO);

    /**
     * 更新画布信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCanvas(@Valid CanvasUpdateReqVO updateReqVO);

    /**
     * 更新画布json信息
     *
     * @param jsonUpdateReqVO 更新信息
     */
    void updateCanvasJson(@Valid CanvasJsonUpdateReqVO jsonUpdateReqVO);

    /**
     * 应用画布
     *
     * @param id 画布id
     */
    void useCanvas(Long id);

    /**
     * 删除画布信息
     *
     * @param id 编号
     */
    void deleteCanvas(Long id);

    /**
     * 获得画布信息
     *
     * @param id 编号
     * @return 画布信息
     */
    CanvasRespVO getCanvas(Long id);

    /**
     * 获得画布明细信息
     *
     * @param reqVO
     * @return 画布信息
     */
    CanvasDetailRespVO getCanvasDetail(CanvasDetailReqVO reqVO);

    /**
     * 获得画布信息列表
     *
     * @param ids 编号
     * @return 画布信息列表
     */
    List<CanvasDO> getCanvasList(Collection<Long> ids);

    /**
     * 获得画布信息分页
     *
     * @param pageReqVO 分页查询
     * @return 画布信息分页
     */
    PageResult<CanvasDO> getCanvasPage(CanvasPageReqVO pageReqVO);

    /**
     * 获得画布信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 画布信息列表
     */
    List<CanvasDO> getCanvasList(CanvasExportReqVO exportReqVO);

}
