package co.yixiang.yshop.module.shop.service.popup;

import java.util.*;
import javax.validation.*;

import co.yixiang.yshop.module.shop.controller.admin.popup.vo.*;
import co.yixiang.yshop.module.shop.controller.app.popup.vo.PopupInfoRespVO;
import co.yixiang.yshop.module.shop.dal.dataobject.popup.PopupDO;
import co.yixiang.yshop.framework.common.pojo.PageResult;

/**
 * 弹窗 Service 接口
 *
 * @author moxiangrong
 */
public interface PopupService {

    /**
     * 创建弹窗
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPopup(@Valid PopupCreateReqVO createReqVO);

    /**
     * 更新弹窗
     *
     * @param updateReqVO 更新信息
     */
    void updatePopup(@Valid PopupUpdateReqVO updateReqVO);

    /**
     * 删除弹窗
     *
     * @param id 编号
     */
    void deletePopup(Long id);

    /**
     * 获得弹窗
     *
     * @param id 编号
     * @return 弹窗
     */
    PopupDO getPopup(Long id);

    /**
     * 获得弹窗列表
     *
     * @param ids 编号
     * @return 弹窗列表
     */
    List<PopupDO> getPopupList(Collection<Long> ids);

    /**
     * 获得弹窗分页
     *
     * @param pageReqVO 分页查询
     * @return 弹窗分页
     */
    PageResult<PopupDO> getPopupPage(PopupPageReqVO pageReqVO);

    /**
     * 状态变更
     *
     * @param id 编号
     * @return 弹窗
     */
    void stateChange(Long id);


    /**
     * 获得当前弹窗信息
     *
     * @return 弹窗
     */
    PopupInfoRespVO getPopupInfo();
}
