package co.yixiang.yshop.module.shop.service.shop;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.shop.controller.admin.shop.vo.*;
import co.yixiang.yshop.module.shop.controller.app.vo.ShopDTO;
import co.yixiang.yshop.module.shop.dal.dataobject.shop.ShopDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 门店 Service 接口
 *
 * @author admin
 */
public interface ShopService {

    /**
     * 创建门店
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShop(@Valid ShopCreateReqVO createReqVO);

    /**
     * 更新门店
     *
     * @param updateReqVO 更新信息
     */
    void updateShop(@Valid ShopUpdateReqVO updateReqVO);

    /**
     * 删除门店
     *
     * @param id 编号
     */
    void deleteShop(Long id);

    /**
     * 获得门店
     *
     * @param id 编号
     * @return 门店
     */
    ShopDO getShop(Long id);

    /**
     * 获得门店列表
     *
     * @param ids 编号
     * @return 门店列表
     */
    List<ShopDO> getShopList(Collection<Long> ids);

    /**
     * 获得门店分页
     *
     * @param pageReqVO 分页查询
     * @return 门店分页
     */
    PageResult<ShopDO> getShopPage(ShopPageReqVO pageReqVO);

    /**
     * 获得门店列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 门店列表
     */
    List<ShopDO> getShopList(ShopExportReqVO exportReqVO);

    /**
     * 客户端根据经纬度查询列表
     *
     * @param dto /
     * @return /
     */
    List<ShopRespVO> getShopList4App(ShopDTO dto);
}
