package co.yixiang.yshop.module.shop.service.shopassistant;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.shop.controller.admin.shopassistant.vo.*;
import co.yixiang.yshop.module.shop.dal.dataobject.shopassistant.ShopAssistantDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 门店店员 Service 接口
 *
 * @author admin
 */
public interface ShopAssistantService {

    /**
     * 创建门店店员
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createShopAssistant(@Valid ShopAssistantCreateReqVO createReqVO);

    /**
     * 更新门店店员
     *
     * @param updateReqVO 更新信息
     */
    void updateShopAssistant(@Valid ShopAssistantUpdateReqVO updateReqVO);

    /**
     * 删除门店店员
     *
     * @param id 编号
     */
    void deleteShopAssistant(Long id);

    /**
     * 获得门店店员
     *
     * @param id 编号
     * @return 门店店员
     */
    ShopAssistantDO getShopAssistant(Long id);

    /**
     * 获得门店店员列表
     *
     * @param ids 编号
     * @return 门店店员列表
     */
    List<ShopAssistantDO> getShopAssistantList(Collection<Long> ids);

    /**
     * 获得门店店员分页
     *
     * @param pageReqVO 分页查询
     * @return 门店店员分页
     */
    PageResult<ShopAssistantRespVO> getShopAssistantPage(ShopAssistantPageReqVO pageReqVO);

    /**
     * 获得门店店员列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 门店店员列表
     */
    List<ShopAssistantDO> getShopAssistantList(ShopAssistantExportReqVO exportReqVO);


    ShopAssistantDO selectByUserId(Long userId);
}
