package co.yixiang.yshop.module.member.dal.mysql.user;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.framework.mybatis.core.mapper.BaseMapperX;
import co.yixiang.yshop.framework.mybatis.core.query.LambdaQueryWrapperX;
import co.yixiang.yshop.module.member.controller.admin.user.vo.UserExportReqVO;
import co.yixiang.yshop.module.member.controller.admin.user.vo.UserPageReqVO;
import co.yixiang.yshop.module.member.dal.dataobject.user.MemberUserDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Update;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.List;

/**
 * 会员 User Mapper
 *
 * @author yshop
 */
@Mapper
public interface MemberUserMapper extends BaseMapperX<MemberUserDO> {

    default MemberUserDO selectByMobile(String mobile) {
        return selectOne(MemberUserDO::getMobile, mobile);
    }

    default List<MemberUserDO> selectListByNicknameLike(String nickname) {
        return selectList(new LambdaQueryWrapperX<MemberUserDO>()
                .likeIfPresent(MemberUserDO::getNickname, nickname));
    }

    default PageResult<MemberUserDO> selectPage(UserPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<MemberUserDO>()
                .likeIfPresent(MemberUserDO::getLoginType, reqVO.getLoginType())
                .likeIfPresent(MemberUserDO::getUsername, reqVO.getUsername())
                .likeIfPresent(MemberUserDO::getRealName, reqVO.getRealName())
                .likeIfPresent(MemberUserDO::getNickname, reqVO.getNickname())
                .likeIfPresent(MemberUserDO::getMobile, reqVO.getMobile())
                .betweenIfPresent(MemberUserDO::getCreateTime, reqVO.getCreateTime())
                .inIfPresent(MemberUserDO::getId, reqVO.getUserIds())
                .orderByDesc(MemberUserDO::getId));
    }

    default List<MemberUserDO> selectList(UserExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<MemberUserDO>()
                .likeIfPresent(MemberUserDO::getUsername, reqVO.getUsername())
                .likeIfPresent(MemberUserDO::getRealName, reqVO.getRealName())
                .likeIfPresent(MemberUserDO::getNickname, reqVO.getNickname())
                .likeIfPresent(MemberUserDO::getMobile, reqVO.getMobile())
                .betweenIfPresent(MemberUserDO::getCreateTime, reqVO.getCreateTime())
                .and(StringUtils.hasText(reqVO.getName()), w ->
                        w.like(MemberUserDO::getUsername, reqVO.getName())
                                .or().like(MemberUserDO::getRealName, reqVO.getName())
                                .or().like(MemberUserDO::getNickname, reqVO.getName())
                                .or().like(MemberUserDO::getMobile, reqVO.getName())
                )
                .orderByDesc(MemberUserDO::getId));
    }

    @Update("update yshop_user set now_money=now_money-#{payPrice}" +
            " where id=#{uid}")
    int decPrice(@Param("payPrice") BigDecimal payPrice, @Param("uid") Long uid);

    @Update("update yshop_user set pay_count=pay_count+1" +
            " where id=#{uid}")
    int incPayCount(@Param("uid") Long uid);

    @Update("update yshop_user set now_money=now_money+#{price}" +
            " where id=#{uid}")
    int incMoney(@Param("uid") Long uid, @Param("price") BigDecimal price);

    @Update("update yshop_user set growth_value=growth_value+#{growthValue}" +
            " where id=#{userId}")
    void addGrowthValue(@Param("userId") Long userId, @Param("growthValue") Integer growthValue);

    @Update("update yshop_user set integral=integral+#{integral}" +
            " where id=#{userId}")
    void addIntegral(@Param("userId") Long userId, @Param("integral") BigDecimal integral);

    @Update("update yshop_user set write_off_authority = #{writeOffAuthority}" +
            " where id=#{userId}")
    void updateWriteOffAuthority(@Param("userId") Long userId, @Param("writeOffAuthority") boolean writeOffAuthority);

    default MemberUserDO getByInvitationCode(String invitationCode) {
        return selectOne(MemberUserDO::getInvitationCode, invitationCode);
    }
}
