package co.yixiang.yshop.module.member.service.integralrule;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.member.controller.admin.integralrule.vo.*;
import co.yixiang.yshop.module.member.dal.dataobject.integralrule.IntegralRuleDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 会员积分规则 Service 接口
 *
 * @author admin
 */
public interface IntegralRuleService {

    /**
     * 创建会员积分规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createIntegralRule(@Valid IntegralRuleCreateReqVO createReqVO);

    /**
     * 更新会员积分规则
     *
     * @param updateReqVO 更新信息
     */
    void updateIntegralRule(@Valid IntegralRuleUpdateReqVO updateReqVO);

    /**
     * 删除会员积分规则
     *
     * @param id 编号
     */
    void deleteIntegralRule(Long id);

    /**
     * 获得会员积分规则
     *
     * @param id 编号
     * @return 会员积分规则
     */
    IntegralRuleDO getIntegralRule(Long id);

    /**
     * 获得会员积分规则列表
     *
     * @param ids 编号
     * @return 会员积分规则列表
     */
    List<IntegralRuleDO> getIntegralRuleList(Collection<Long> ids);

    /**
     * 获得会员积分规则分页
     *
     * @param pageReqVO 分页查询
     * @return 会员积分规则分页
     */
    PageResult<IntegralRuleDO> getIntegralRulePage(IntegralRulePageReqVO pageReqVO);

    /**
     * 获得会员积分规则列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 会员积分规则列表
     */
    List<IntegralRuleDO> getIntegralRuleList(IntegralRuleExportReqVO exportReqVO);

    /**
     * 根据类型列表查询
     *
     * @param typeList /
     * @return /
     */
    List<IntegralRuleDO> getIntegralRuleByTypes(List<String> typeList);

    /**
     * 查询会员积分规则
     *
     * @return /
     */
    IntegralRuleVO getRule();

    /**
     * 保存会员积分规则
     *
     * @param vo /
     */
    void saveRule(IntegralRuleVO vo);
}
