package co.yixiang.yshop.module.member.service.signinrecord;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.member.controller.admin.signinrecord.vo.SignInRecordCreateReqVO;
import co.yixiang.yshop.module.member.controller.admin.signinrecord.vo.SignInRecordExportReqVO;
import co.yixiang.yshop.module.member.controller.admin.signinrecord.vo.SignInRecordPageReqVO;
import co.yixiang.yshop.module.member.controller.admin.signinrecord.vo.SignInRecordUpdateReqVO;
import co.yixiang.yshop.module.member.controller.app.signinrecord.vo.SignInInfoVO;
import co.yixiang.yshop.module.member.dal.dataobject.signinrecord.SignInRecordDO;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.List;

/**
 * 签到记录 Service 接口
 *
 * @author admin
 */
public interface SignInRecordService {

    /**
     * 创建签到记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSignInRecord(@Valid SignInRecordCreateReqVO createReqVO);

    /**
     * 更新签到记录
     *
     * @param updateReqVO 更新信息
     */
    void updateSignInRecord(@Valid SignInRecordUpdateReqVO updateReqVO);

    /**
     * 删除签到记录
     *
     * @param id 编号
     */
    void deleteSignInRecord(Long id);

    /**
     * 获得签到记录
     *
     * @param id 编号
     * @return 签到记录
     */
    SignInRecordDO getSignInRecord(Long id);

    /**
     * 获得签到记录列表
     *
     * @param ids 编号
     * @return 签到记录列表
     */
    List<SignInRecordDO> getSignInRecordList(Collection<Long> ids);

    /**
     * 获得签到记录分页
     *
     * @param pageReqVO 分页查询
     * @return 签到记录分页
     */
    PageResult<SignInRecordDO> getSignInRecordPage(SignInRecordPageReqVO pageReqVO);

    /**
     * 获得签到记录列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 签到记录列表
     */
    List<SignInRecordDO> getSignInRecordList(SignInRecordExportReqVO exportReqVO);

    /**
     * 获取用户连续签到次数
     *
     * @param userId /
     * @return /
     */
    Integer countByUserIdThisWeek(Long userId);

    /**
     * 签到信息
     *
     * @param loginUserId /
     * @param startTime   开始时间
     * @param endTime     结束时间
     * @return /
     */
    SignInInfoVO signInInfo(Long loginUserId, LocalDateTime startTime, LocalDateTime endTime);

    /**
     * 签到
     *
     * @param loginUserId /
     * @return /
     */
    BigDecimal signIn(Long loginUserId);


}
