package co.yixiang.yshop.module.member.service.userinvitelog;

import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.member.controller.admin.userinvitelog.vo.UserInviteLogCreateReqVO;
import co.yixiang.yshop.module.member.controller.admin.userinvitelog.vo.UserInviteLogExportReqVO;
import co.yixiang.yshop.module.member.controller.admin.userinvitelog.vo.UserInviteLogPageReqVO;
import co.yixiang.yshop.module.member.controller.admin.userinvitelog.vo.UserInviteLogUpdateReqVO;
import co.yixiang.yshop.module.member.convert.userinvitelog.UserInviteLogConvert;
import co.yixiang.yshop.module.member.dal.dataobject.userinvitelog.UserInviteLogDO;
import co.yixiang.yshop.module.member.dal.mysql.userinvitelog.UserInviteLogMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static co.yixiang.yshop.framework.common.exception.util.ServiceExceptionUtil.exception;
import static co.yixiang.yshop.module.member.enums.ErrorCodeConstants.USER_INVITE_LOG_NOT_EXISTS;

/**
 * 邀请日志 Service 实现类
 *
 * @author admin
 */
@Service
@Validated
public class UserInviteLogServiceImpl implements UserInviteLogService {

    @Resource
    private UserInviteLogMapper userInviteLogMapper;

    @Override
    public Long createUserInviteLog(UserInviteLogCreateReqVO createReqVO) {
        // 插入
        UserInviteLogDO userInviteLog = UserInviteLogConvert.INSTANCE.convert(createReqVO);
        userInviteLogMapper.insert(userInviteLog);
        // 返回
        return userInviteLog.getId();
    }

    @Override
    public void updateUserInviteLog(UserInviteLogUpdateReqVO updateReqVO) {
        // 校验存在
        validateUserInviteLogExists(updateReqVO.getId());
        // 更新
        UserInviteLogDO updateObj = UserInviteLogConvert.INSTANCE.convert(updateReqVO);
        userInviteLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteUserInviteLog(Long id) {
        // 校验存在
        validateUserInviteLogExists(id);
        // 删除
        userInviteLogMapper.deleteById(id);
    }

    private void validateUserInviteLogExists(Long id) {
        if (userInviteLogMapper.selectById(id) == null) {
            throw exception(USER_INVITE_LOG_NOT_EXISTS);
        }
    }

    @Override
    public UserInviteLogDO getUserInviteLog(Long id) {
        return userInviteLogMapper.selectById(id);
    }

    @Override
    public List<UserInviteLogDO> getUserInviteLogList(Collection<Long> ids) {
        return userInviteLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserInviteLogDO> getUserInviteLogPage(UserInviteLogPageReqVO pageReqVO) {
        return userInviteLogMapper.selectPage(pageReqVO);
    }

    @Override
    public List<UserInviteLogDO> getUserInviteLogList(UserInviteLogExportReqVO exportReqVO) {
        return userInviteLogMapper.selectList(exportReqVO);
    }

    @Override
    public void createUserInviteLog(Long userId, Long beInviteUserId) {
        UserInviteLogDO log = new UserInviteLogDO();
        log.setUserId(userId);
        log.setBeInviteUserId(beInviteUserId);
        userInviteLogMapper.insert(log);
    }

}
