package co.yixiang.yshop.module.member.service.userlevelconfig;

import java.util.*;
import javax.validation.*;

import co.yixiang.yshop.module.member.controller.admin.userlevelconfig.vo.*;
import co.yixiang.yshop.module.member.controller.app.userlevel.vo.SettleLevelInfoVO;
import co.yixiang.yshop.module.member.controller.app.userlevel.vo.UserLevelInfoVO;
import co.yixiang.yshop.module.member.dal.dataobject.userlevelconfig.UserLevelConfigDO;
import co.yixiang.yshop.framework.common.pojo.PageResult;
import co.yixiang.yshop.module.member.dal.dataobject.userlevelequity.UserLevelEquityDO;

/**
 * 会员等级配置 Service 接口
 *
 * @author admin
 */
public interface UserLevelConfigService {

    /**
     * 创建会员等级配置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createUserLevelConfig(@Valid UserLevelConfigCreateReqVO createReqVO);

    /**
     * 更新会员等级配置
     *
     * @param updateReqVO 更新信息
     */
    void updateUserLevelConfig(@Valid UserLevelConfigUpdateReqVO updateReqVO);

    /**
     * 删除会员等级配置
     *
     * @param id 编号
     */
    void deleteUserLevelConfig(Long id);

    /**
     * 获得会员等级配置
     *
     * @param id 编号
     * @return 会员等级配置
     */
    UserLevelConfigDO getUserLevelConfig(Long id);

    /**
     * 获得会员等级配置列表
     *
     * @param ids 编号
     * @return 会员等级配置列表
     */
    List<UserLevelConfigDO> getUserLevelConfigList(Collection<Long> ids);

    /**
     * 获得会员等级配置分页
     *
     * @param pageReqVO 分页查询
     * @return 会员等级配置分页
     */
    PageResult<UserLevelConfigDO> getUserLevelConfigPage(UserLevelConfigPageReqVO pageReqVO);

    /**
     * 获得会员等级配置列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 会员等级配置列表
     */
    List<UserLevelConfigDO> getUserLevelConfigList(UserLevelConfigExportReqVO exportReqVO);

    /**
     * 用户等级信息
     *
     * @param loginUserId /
     * @return /
     */
    UserLevelInfoVO levelInfo(Long loginUserId);

    /**
     * 升级
     *
     * @param userId /
     */
    void upgradeLevel(Long userId);

    /**
     * 查询等级详情
     *
     * @param id /
     * @return /
     */
    UserLevelConfigRespVO getUserLevelConfigVO(Long id);

    /**
     * 当前等级权益
     *
     * @param loginUserId /
     * @return /
     */
    List<UserLevelEquityDO> getUserLevelEquityListByUserId(Long loginUserId);


    /**
     * 结算时等级信息
     *
     * @param loginUserId /
     * @return /
     */
    SettleLevelInfoVO settleLevelInfo(Long loginUserId);
}
