package co.yixiang.yshop.module.member.service.userinvitelog;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import co.yixiang.yshop.framework.test.core.ut.BaseDbUnitTest;

import co.yixiang.yshop.module.member.controller.admin.userinvitelog.vo.*;
import co.yixiang.yshop.module.member.dal.dataobject.userinvitelog.UserInviteLogDO;
import co.yixiang.yshop.module.member.dal.mysql.userinvitelog.UserInviteLogMapper;
import co.yixiang.yshop.framework.common.pojo.PageResult;

import javax.annotation.Resource;

import org.springframework.context.annotation.Import;

import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static co.yixiang.yshop.module.member.enums.ErrorCodeConstants.*;
import static co.yixiang.yshop.framework.test.core.util.AssertUtils.*;
import static co.yixiang.yshop.framework.test.core.util.RandomUtils.*;
import static co.yixiang.yshop.framework.common.util.date.LocalDateTimeUtils.*;
import static co.yixiang.yshop.framework.common.util.object.ObjectUtils.*;
import static co.yixiang.yshop.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link UserInviteLogServiceImpl} 的单元测试类
 *
 * @author admin
 */
@Import(UserInviteLogServiceImpl.class)
public class UserInviteLogServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UserInviteLogServiceImpl userInviteLogService;

    @Resource
    private UserInviteLogMapper userInviteLogMapper;

    @Test
    public void testCreateUserInviteLog_success() {
        // 准备参数
        UserInviteLogCreateReqVO reqVO = randomPojo(UserInviteLogCreateReqVO.class);

        // 调用
        Long userInviteLogId = userInviteLogService.createUserInviteLog(reqVO);
        // 断言
        assertNotNull(userInviteLogId);
        // 校验记录的属性是否正确
        UserInviteLogDO userInviteLog = userInviteLogMapper.selectById(userInviteLogId);
        assertPojoEquals(reqVO, userInviteLog);
    }

    @Test
    public void testUpdateUserInviteLog_success() {
        // mock 数据
        UserInviteLogDO dbUserInviteLog = randomPojo(UserInviteLogDO.class);
        userInviteLogMapper.insert(dbUserInviteLog);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UserInviteLogUpdateReqVO reqVO = randomPojo(UserInviteLogUpdateReqVO.class, o -> {
            o.setId(dbUserInviteLog.getId()); // 设置更新的 ID
        });

        // 调用
        userInviteLogService.updateUserInviteLog(reqVO);
        // 校验是否更新正确
        UserInviteLogDO userInviteLog = userInviteLogMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, userInviteLog);
    }

    @Test
    public void testUpdateUserInviteLog_notExists() {
        // 准备参数
        UserInviteLogUpdateReqVO reqVO = randomPojo(UserInviteLogUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> userInviteLogService.updateUserInviteLog(reqVO), USER_INVITE_LOG_NOT_EXISTS);
    }

    @Test
    public void testDeleteUserInviteLog_success() {
        // mock 数据
        UserInviteLogDO dbUserInviteLog = randomPojo(UserInviteLogDO.class);
        userInviteLogMapper.insert(dbUserInviteLog);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUserInviteLog.getId();

        // 调用
        userInviteLogService.deleteUserInviteLog(id);
        // 校验数据不存在了
        assertNull(userInviteLogMapper.selectById(id));
    }

    @Test
    public void testDeleteUserInviteLog_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> userInviteLogService.deleteUserInviteLog(id), USER_INVITE_LOG_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserInviteLogPage() {
        // mock 数据
        UserInviteLogDO dbUserInviteLog = randomPojo(UserInviteLogDO.class, o -> { // 等会查询到
            o.setUserId(null);
            o.setBeInviteUserId(null);
            o.setCreateTime(null);
        });
        userInviteLogMapper.insert(dbUserInviteLog);
        // 测试 userId 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setUserId(null)));
        // 测试 beInviteUserId 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setBeInviteUserId(null)));
        // 测试 createTime 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setCreateTime(null)));
        // 准备参数
        UserInviteLogPageReqVO reqVO = new UserInviteLogPageReqVO();
        reqVO.setUserId(null);
        reqVO.setBeInviteUserId(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        PageResult<UserInviteLogDO> pageResult = userInviteLogService.getUserInviteLogPage(reqVO);
        // 断言
        assertEquals(1, pageResult.getTotal());
        assertEquals(1, pageResult.getList().size());
        assertPojoEquals(dbUserInviteLog, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserInviteLogList() {
        // mock 数据
        UserInviteLogDO dbUserInviteLog = randomPojo(UserInviteLogDO.class, o -> { // 等会查询到
            o.setUserId(null);
            o.setBeInviteUserId(null);
            o.setCreateTime(null);
        });
        userInviteLogMapper.insert(dbUserInviteLog);
        // 测试 userId 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setUserId(null)));
        // 测试 beInviteUserId 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setBeInviteUserId(null)));
        // 测试 createTime 不匹配
        userInviteLogMapper.insert(cloneIgnoreId(dbUserInviteLog, o -> o.setCreateTime(null)));
        // 准备参数
        UserInviteLogExportReqVO reqVO = new UserInviteLogExportReqVO();
        reqVO.setUserId(null);
        reqVO.setBeInviteUserId(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        List<UserInviteLogDO> list = userInviteLogService.getUserInviteLogList(reqVO);
        // 断言
        assertEquals(1, list.size());
        assertPojoEquals(dbUserInviteLog, list.get(0));
    }

}
