package co.yixiang.yshop.module.member.service.userlevelconfig;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import co.yixiang.yshop.framework.test.core.ut.BaseDbUnitTest;

import co.yixiang.yshop.module.member.controller.admin.userlevelconfig.vo.*;
import co.yixiang.yshop.module.member.dal.dataobject.userlevelconfig.UserLevelConfigDO;
import co.yixiang.yshop.module.member.dal.mysql.userlevelconfig.UserLevelConfigMapper;
import co.yixiang.yshop.framework.common.pojo.PageResult;

import javax.annotation.Resource;

import org.springframework.context.annotation.Import;

import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static co.yixiang.yshop.module.member.enums.ErrorCodeConstants.*;
import static co.yixiang.yshop.framework.test.core.util.AssertUtils.*;
import static co.yixiang.yshop.framework.test.core.util.RandomUtils.*;
import static co.yixiang.yshop.framework.common.util.date.LocalDateTimeUtils.*;
import static co.yixiang.yshop.framework.common.util.object.ObjectUtils.*;
import static co.yixiang.yshop.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link UserLevelConfigServiceImpl} 的单元测试类
 *
 * @author admin
 */
@Import(UserLevelConfigServiceImpl.class)
public class UserLevelConfigServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UserLevelConfigServiceImpl userLevelConfigService;

    @Resource
    private UserLevelConfigMapper userLevelConfigMapper;

    @Test
    public void testCreateUserLevelConfig_success() {
        // 准备参数
        UserLevelConfigCreateReqVO reqVO = randomPojo(UserLevelConfigCreateReqVO.class);

        // 调用
        Long userLevelConfigId = userLevelConfigService.createUserLevelConfig(reqVO);
        // 断言
        assertNotNull(userLevelConfigId);
        // 校验记录的属性是否正确
        UserLevelConfigDO userLevelConfig = userLevelConfigMapper.selectById(userLevelConfigId);
        assertPojoEquals(reqVO, userLevelConfig);
    }

    @Test
    public void testUpdateUserLevelConfig_success() {
        // mock 数据
        UserLevelConfigDO dbUserLevelConfig = randomPojo(UserLevelConfigDO.class);
        userLevelConfigMapper.insert(dbUserLevelConfig);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UserLevelConfigUpdateReqVO reqVO = randomPojo(UserLevelConfigUpdateReqVO.class, o -> {
            o.setId(dbUserLevelConfig.getId()); // 设置更新的 ID
        });

        // 调用
        userLevelConfigService.updateUserLevelConfig(reqVO);
        // 校验是否更新正确
        UserLevelConfigDO userLevelConfig = userLevelConfigMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, userLevelConfig);
    }

    @Test
    public void testUpdateUserLevelConfig_notExists() {
        // 准备参数
        UserLevelConfigUpdateReqVO reqVO = randomPojo(UserLevelConfigUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> userLevelConfigService.updateUserLevelConfig(reqVO), USER_LEVEL_CONFIG_NOT_EXISTS);
    }

    @Test
    public void testDeleteUserLevelConfig_success() {
        // mock 数据
        UserLevelConfigDO dbUserLevelConfig = randomPojo(UserLevelConfigDO.class);
        userLevelConfigMapper.insert(dbUserLevelConfig);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUserLevelConfig.getId();

        // 调用
        userLevelConfigService.deleteUserLevelConfig(id);
        // 校验数据不存在了
        assertNull(userLevelConfigMapper.selectById(id));
    }

    @Test
    public void testDeleteUserLevelConfig_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> userLevelConfigService.deleteUserLevelConfig(id), USER_LEVEL_CONFIG_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserLevelConfigPage() {
        // mock 数据
        UserLevelConfigDO dbUserLevelConfig = randomPojo(UserLevelConfigDO.class, o -> { // 等会查询到
            o.setLevelName(null);
            o.setLevel(null);
            o.setGrowthValue(null);
            o.setIconUrl(null);
            o.setBackgroundUrl(null);
            o.setStatus(null);
            o.setRemarks(null);
            o.setCreateTime(null);
        });
        userLevelConfigMapper.insert(dbUserLevelConfig);
        // 测试 levelName 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setLevelName(null)));
        // 测试 level 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setLevel(null)));
        // 测试 growthValue 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setGrowthValue(null)));
        // 测试 iconUrl 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setIconUrl(null)));
        // 测试 backgroundUrl 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setBackgroundUrl(null)));
        // 测试 status 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setStatus(null)));
        // 测试 remarks 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setRemarks(null)));
        // 测试 createTime 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setCreateTime(null)));
        // 准备参数
        UserLevelConfigPageReqVO reqVO = new UserLevelConfigPageReqVO();
        reqVO.setLevelName(null);
        reqVO.setLevel(null);
        reqVO.setGrowthValue(null);
        reqVO.setIconUrl(null);
        reqVO.setBackgroundUrl(null);
        reqVO.setStatus(null);
        reqVO.setRemarks(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        PageResult<UserLevelConfigDO> pageResult = userLevelConfigService.getUserLevelConfigPage(reqVO);
        // 断言
        assertEquals(1, pageResult.getTotal());
        assertEquals(1, pageResult.getList().size());
        assertPojoEquals(dbUserLevelConfig, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserLevelConfigList() {
        // mock 数据
        UserLevelConfigDO dbUserLevelConfig = randomPojo(UserLevelConfigDO.class, o -> { // 等会查询到
            o.setLevelName(null);
            o.setLevel(null);
            o.setGrowthValue(null);
            o.setIconUrl(null);
            o.setBackgroundUrl(null);
            o.setStatus(null);
            o.setRemarks(null);
            o.setCreateTime(null);
        });
        userLevelConfigMapper.insert(dbUserLevelConfig);
        // 测试 levelName 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setLevelName(null)));
        // 测试 level 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setLevel(null)));
        // 测试 growthValue 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setGrowthValue(null)));
        // 测试 iconUrl 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setIconUrl(null)));
        // 测试 backgroundUrl 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setBackgroundUrl(null)));
        // 测试 status 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setStatus(null)));
        // 测试 remarks 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setRemarks(null)));
        // 测试 createTime 不匹配
        userLevelConfigMapper.insert(cloneIgnoreId(dbUserLevelConfig, o -> o.setCreateTime(null)));
        // 准备参数
        UserLevelConfigExportReqVO reqVO = new UserLevelConfigExportReqVO();
        reqVO.setLevelName(null);
        reqVO.setLevel(null);
        reqVO.setGrowthValue(null);
        reqVO.setIconUrl(null);
        reqVO.setBackgroundUrl(null);
        reqVO.setStatus(null);
        reqVO.setRemarks(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        List<UserLevelConfigDO> list = userLevelConfigService.getUserLevelConfigList(reqVO);
        // 断言
        assertEquals(1, list.size());
        assertPojoEquals(dbUserLevelConfig, list.get(0));
    }

}
