package co.yixiang.yshop.module.member.service.usertag;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import co.yixiang.yshop.framework.test.core.ut.BaseDbUnitTest;

import co.yixiang.yshop.module.member.controller.admin.usertag.vo.*;
import co.yixiang.yshop.module.member.dal.dataobject.usertag.UserTagDO;
import co.yixiang.yshop.module.member.dal.mysql.usertag.UserTagMapper;
import co.yixiang.yshop.framework.common.pojo.PageResult;

import javax.annotation.Resource;

import org.springframework.context.annotation.Import;

import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static co.yixiang.yshop.module.member.enums.ErrorCodeConstants.*;
import static co.yixiang.yshop.framework.test.core.util.AssertUtils.*;
import static co.yixiang.yshop.framework.test.core.util.RandomUtils.*;
import static co.yixiang.yshop.framework.common.util.date.LocalDateTimeUtils.*;
import static co.yixiang.yshop.framework.common.util.object.ObjectUtils.*;
import static co.yixiang.yshop.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link UserTagServiceImpl} 的单元测试类
 *
 * @author admin
 */
@Import(UserTagServiceImpl.class)
public class UserTagServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UserTagServiceImpl userTagService;

    @Resource
    private UserTagMapper userTagMapper;

    @Test
    public void testCreateUserTag_success() {
        // 准备参数
        UserTagCreateReqVO reqVO = randomPojo(UserTagCreateReqVO.class);

        // 调用
        Long userTagId = userTagService.createUserTag(reqVO);
        // 断言
        assertNotNull(userTagId);
        // 校验记录的属性是否正确
        UserTagDO userTag = userTagMapper.selectById(userTagId);
        assertPojoEquals(reqVO, userTag);
    }

    @Test
    public void testUpdateUserTag_success() {
        // mock 数据
        UserTagDO dbUserTag = randomPojo(UserTagDO.class);
        userTagMapper.insert(dbUserTag);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UserTagUpdateReqVO reqVO = randomPojo(UserTagUpdateReqVO.class, o -> {
            o.setId(dbUserTag.getId()); // 设置更新的 ID
        });

        // 调用
        userTagService.updateUserTag(reqVO);
        // 校验是否更新正确
        UserTagDO userTag = userTagMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, userTag);
    }

    @Test
    public void testUpdateUserTag_notExists() {
        // 准备参数
        UserTagUpdateReqVO reqVO = randomPojo(UserTagUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> userTagService.updateUserTag(reqVO), USER_TAG_NOT_EXISTS);
    }

    @Test
    public void testDeleteUserTag_success() {
        // mock 数据
        UserTagDO dbUserTag = randomPojo(UserTagDO.class);
        userTagMapper.insert(dbUserTag);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUserTag.getId();

        // 调用
        userTagService.deleteUserTag(id);
        // 校验数据不存在了
        assertNull(userTagMapper.selectById(id));
    }

    @Test
    public void testDeleteUserTag_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> userTagService.deleteUserTag(id), USER_TAG_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserTagPage() {
        // mock 数据
        UserTagDO dbUserTag = randomPojo(UserTagDO.class, o -> { // 等会查询到
            o.setTagName(null);
            o.setGroupId(null);
            o.setCreateTime(null);
        });
        userTagMapper.insert(dbUserTag);
        // 测试 tagName 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setTagName(null)));
        // 测试 groupId 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setGroupId(null)));
        // 测试 createTime 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setCreateTime(null)));
        // 准备参数
        UserTagPageReqVO reqVO = new UserTagPageReqVO();
        reqVO.setTagName(null);
        reqVO.setGroupId(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        PageResult<UserTagDO> pageResult = userTagService.getUserTagPage(reqVO);
        // 断言
        assertEquals(1, pageResult.getTotal());
        assertEquals(1, pageResult.getList().size());
        assertPojoEquals(dbUserTag, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserTagList() {
        // mock 数据
        UserTagDO dbUserTag = randomPojo(UserTagDO.class, o -> { // 等会查询到
            o.setTagName(null);
            o.setGroupId(null);
            o.setCreateTime(null);
        });
        userTagMapper.insert(dbUserTag);
        // 测试 tagName 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setTagName(null)));
        // 测试 groupId 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setGroupId(null)));
        // 测试 createTime 不匹配
        userTagMapper.insert(cloneIgnoreId(dbUserTag, o -> o.setCreateTime(null)));
        // 准备参数
        UserTagExportReqVO reqVO = new UserTagExportReqVO();
        reqVO.setTagName(null);
        reqVO.setGroupId(null);
        reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

        // 调用
        List<UserTagDO> list = userTagService.getUserTagList(reqVO);
        // 断言
        assertEquals(1, list.size());
        assertPojoEquals(dbUserTag, list.get(0));
    }

}
